@@ -60,23 +60,17 @@ DEFAULT_ROLE_SETUP_PARAMS = {
 
 def add_signing_key(
     repo_path,
-    role,
+    roles,
     pub_key_path=None,
     keystore=None,
     roles_key_infos=None,
     scheme=DEFAULT_RSA_SIGNATURE_SCHEME,
 ):
     """
-    Adds a new signing key. Currently assumes that root keys, as well as signing key of the role
-    whose signing key is being added are stored on yubikeys. Automatically updates timestamp and
-    snapshot. Timestamp and snapshot keys can be loaded from the filesystem if located inside
-    the keystore whose location is specified via either the keystore argument, or role_key_infos
-    json.
+    Adds a new signing key to the listed roles. Automatically updates timestamp and
+    snapshot.
     """
-    from taf.repository_tool import yubikey_signature_provider
-
     taf_repo = Repository(repo_path)
-
     roles_key_infos, keystore = _initialize_roles_and_keystore(
         roles_key_infos, keystore, enter_info=False
     )
@@ -91,50 +85,28 @@ def add_signing_key(
     if pub_key_pem is None:
         pub_key_pem = new_public_key_cmd_prompt(scheme)["keyval"]["public"]
 
-    if taf_repo.is_valid_metadata_key(role, pub_key_pem):
-        print(f"Key already registered as signing key of role {role}")
+    parent_roles = set()
+    for role in roles:
+
+        if taf_repo.is_valid_metadata_key(role, pub_key_pem):
+            print(f"Key already registered as signing key of role {role}")
+            continue
+
+        taf_repo.add_metadata_key(role, pub_key_pem, scheme)
+
+        if is_delegated_role(role):
+            parent_role = taf_repo.find_delegated_roles_parent(role)
+        else:
+            parent_role = "root"
+
+        parent_roles.add(parent_role)
+
+    if not len(parent_roles):
         return
 
-    taf_repo.add_metadata_key(role, pub_key_pem, scheme)
-
-    if is_delegated_role(role):
-        parent_role = taf_repo.find_delegated_roles_parent(role)
-    else:
-        parent_role = "root"
-
-    parent_obj = taf_repo._role_obj(parent_role)
-
-    threshold = parent_obj.threshold
-    keys_num = len(parent_obj.keys)
-    num_of_signatures = 0
-    loaded_yubikeys = {}
-    pub_key, _ = yk.yubikey_prompt(
-        role, role, taf_repo, loaded_yubikeys=loaded_yubikeys
-    )
-    role_obj = _role_obj(role, taf_repo._repository)
-    role_obj.add_external_signature_provider(
-        pub_key, partial(yubikey_signature_provider, role, pub_key["keyid"])
-    )
-
-    all_loaded = False
-    while not all_loaded:
-        if num_of_signatures >= threshold:
-            all_loaded = not (
-                click.confirm(
-                    f"Threshold of {parent_role} keys reached. Do you want to load more root keys?"
-                )
-            )
-        if not all_loaded:
-            name = f"{parent_role}{num_of_signatures+1}"
-            pub_key, _ = yk.yubikey_prompt(
-                name, parent_role, taf_repo, loaded_yubikeys=loaded_yubikeys
-            )
-            parent_obj.add_external_signature_provider(
-                pub_key, partial(yubikey_signature_provider, name, pub_key["keyid"])
-            )
-            num_of_signatures += 1
-        if num_of_signatures == keys_num:
-            all_loaded = True
+    taf_repo.unmark_dirty_roles(list(set(roles) - parent_roles))
+    for parent_role in parent_roles:
+        _update_role(taf_repo, parent_role, keystore, roles_infos, scheme)
 
     update_snapshot_and_timestamp(taf_repo, keystore, roles_infos, scheme=scheme)
 
@@ -770,6 +742,26 @@ def export_yk_certificate(certs_dir, key):
         f.write(yk.export_piv_x509())
 
 
+def export_targets_history(repo_path, commit=None, output=None, target_repos=None):
+    auth_repo = AuthenticationRepo(repo_path)
+    commits = auth_repo.all_commits_since_commit(commit, branch="master")
+    if not len(target_repos):
+        target_repos = None
+    commits_on_branches = auth_repo.sorted_commits_and_branches_per_repositories(
+        commits, target_repos
+    )
+    commits_json = json.dumps(commits_on_branches, indent=4)
+    if output is not None:
+        output = Path(output).resolve()
+        if output.suffix != ".json":
+            output = output.with_suffix(".json")
+        output.parent.mkdir(parents=True, exist_ok=True)
+        output.write_text(commits_json)
+        print(f"Result written to {output}")
+    else:
+        print(commits_json)
+
+
 def _get_namespace_and_root(repo_path, namespace, root_dir):
     repo_path = Path(repo_path).resolve()
     if namespace is None:
@@ -823,6 +815,7 @@ def generate_repositories_json(
     namespace=None,
     targets_relative_dir=None,
     custom_data=None,
+    use_mirrors=True,
 ):
     """
     <Purpose>
@@ -837,13 +830,17 @@ def generate_repositories_json(
         is expected to be root_dir/namespace directory
         targets_relative_dir:
         Directory relative to which urls of the target repositories are set, if they do not have remote set
-        custom_date:
+        custom_data:
         Dictionary or path to a json file containing additional information about the repositories that
         should be added to repositories.json
+        use_mirrors:
+        Determines whether to generate mirror.json, which contains a list of mirror templates, or
+        to generate url elements in repositories.json
     """
 
     custom_data = read_input_dict(custom_data)
     repositories = {}
+    mirrors = []
     repo_path = Path(repo_path).resolve()
     auth_repo_targets_dir = repo_path / TARGETS_DIRECTORY_NAME
     # if targets directory is not specified, assume that target repositories
@@ -854,6 +851,7 @@ def generate_repositories_json(
         targets_relative_dir = Path(targets_relative_dir).resolve()
 
     print(f"Adding all repositories from {targets_directory}")
+
     for target_repo_dir in targets_directory.glob("*"):
         if not target_repo_dir.is_dir() or target_repo_dir == repo_path:
             continue
@@ -876,7 +874,16 @@ def generate_repositories_json(
                 url = Path(target_repo.path).resolve()
             # convert to posix path
             url = str(url.as_posix())
-        repositories[target_repo_namespaced_name] = {"urls": [url]}
+
+        if use_mirrors:
+            url = url.replace(namespace, "{org_name}").replace(
+                target_repo_name, "{repo_name}"
+            )
+            mirrors.append(url)
+            repositories[target_repo_namespaced_name] = {}
+        else:
+            repositories[target_repo_namespaced_name] = {"urls": [url]}
+
         if target_repo_namespaced_name in custom_data:
             repositories[target_repo_namespaced_name]["custom"] = custom_data[
                 target_repo_namespaced_name
@@ -885,6 +892,10 @@ def generate_repositories_json(
     file_path = auth_repo_targets_dir / "repositories.json"
     file_path.write_text(json.dumps({"repositories": repositories}, indent=4))
     print(f"Generated {file_path}")
+    if use_mirrors:
+        mirrors_path = auth_repo_targets_dir / "mirrors.json"
+        mirrors_path.write_text(json.dumps({"mirrors": mirrors}, indent=4))
+        print(f"Generated {mirrors_path}")
 
 
 def _get_key_name(role_name, key_num, num_of_keys):
@@ -900,6 +911,7 @@ def init_repo(
     namespace=None,
     targets_relative_dir=None,
     custom_data=None,
+    use_mirrors=True,
     add_branch=None,
     keystore=None,
     roles_key_infos=None,
@@ -926,9 +938,12 @@ def init_repo(
         is expected to be root_dir/namespace directory
         targets_relative_dir:
         Directory relative to which urls of the target repositories are set, if they do not have remote set
-        custom_date:
+        custom_data:
         Dictionary or path to a json file containing additional information about the repositories that
         should be added to repositories.json
+        use_mirrors:
+        Determines whether to generate mirror.json, which contains a list of mirror templates, or
+        to generate url elements in repositories.json.
         add_branch:
         Indicates whether to add the current branch's name to the target file
         keystore:
@@ -950,7 +965,7 @@ def init_repo(
     create_repository(repo_path, keystore, roles_key_infos, commit, test)
     update_target_repos_from_fs(repo_path, targets_directory, namespace, add_branch)
     generate_repositories_json(
-        repo_path, root_dir, namespace, targets_relative_dir, custom_data
+        repo_path, root_dir, namespace, targets_relative_dir, custom_data, use_mirrors
     )
     register_target_files(repo_path, keystore, roles_key_infos, commit, scheme=scheme)
 
@@ -987,7 +1002,6 @@ def register_target_files(
         roles_key_infos, keystore, enter_info=False
     )
     roles_infos = roles_key_infos.get("roles")
-
     if taf_repo is None:
         repo_path = Path(repo_path).resolve()
         taf_repo = Repository(str(repo_path))
@@ -1118,7 +1132,7 @@ def update_metadata_expiration_date(
             taf_repo.update_role_keystores(
                 role, keys, start_date=start_date, interval=interval
             )
-        else:  # sign with yubikey
+        if len(yubikeys):  # sign with yubikey
             taf_repo.update_role_yubikeys(
                 role, yubikeys, start_date=start_date, interval=interval
             )
@@ -1134,6 +1148,16 @@ def update_metadata_expiration_date(
         auth_repo.commit(commit_message)
 
 
+def _update_role(taf_repo, role, keystore, roles_infos, scheme):
+    keystore_keys, yubikeys = _load_signing_keys(
+        taf_repo, role, keystore, roles_infos, scheme=scheme
+    )
+    if len(keystore_keys):
+        taf_repo.update_role_keystores(role, keystore_keys, write=False)
+    if len(yubikeys):
+        taf_repo.update_role_yubikeys(role, yubikeys, write=False)
+
+
 def update_snapshot_and_timestamp(
     taf_repo, keystore, roles_infos, scheme=DEFAULT_RSA_SIGNATURE_SCHEME, write_all=True
 ):
@@ -1146,7 +1170,7 @@ def update_snapshot_and_timestamp(
         if len(yubikeys):
             update_method = taf_repo.roles_yubikeys_update_method(role)
             update_method(yubikeys, write=False)
-        else:
+        if len(keystore_keys):
             update_method = taf_repo.roles_keystore_update_method(role)
             update_method(keystore_keys, write=False)
 
@@ -1198,7 +1222,7 @@ def _update_target_roles(
 
         if len(yubikeys):
             taf_repo.update_targets_yubikeys(yubikeys, write=False, **targets_data)
-        else:
+        if len(keystore_keys):
             taf_repo.update_targets_keystores(
                 keystore_keys, write=False, **targets_data
             )
