@@ -9,7 +9,7 @@ from pathlib import Path
 
 import securesystemslib
 import tuf.roledb
-from securesystemslib.exceptions import Error as SSLibError
+from securesystemslib.exceptions import Error as SSLibError, RepositoryError
 from securesystemslib.interface import import_rsa_privatekey_from_file
 from securesystemslib.util import HASH_FUNCTION, get_file_details
 from tuf.exceptions import Error as TUFError
@@ -33,6 +33,7 @@ from taf.exceptions import (
     TargetsMetadataUpdateError,
     TimestampMetadataUpdateError,
     YubikeyError,
+    InvalidRepositoryError,
 )
 from taf.git import GitRepository
 from taf.utils import normalize_file_line_endings, on_rm_error
@@ -192,6 +193,30 @@ class Repository:
         certs_dir.mkdir(parents=True, exist_ok=True)
         return str(certs_dir)
 
+    def _add_delegated_key(
+        self, role, keyid, pub_key, keytype="rsa", scheme=DEFAULT_RSA_SIGNATURE_SCHEME
+    ):
+        """
+        Adds public key of a new delegated role to the list of all keys of
+        delegated roles.
+        Args:
+        - role (str): parent target role's name
+        - keyid (str): keyid of the new signing key
+        - pub_key(str): public component of the new signing key
+        - keytype (str): key's type
+        - sheme (str): signature scheme
+        """
+        roleinfo = tuf.roledb.get_roleinfo(role, self.name)
+        keysinfo = roleinfo["delegations"]["keys"]
+        if keyid in keysinfo:
+            return
+        key = {"public": pub_key.strip()}
+        key_metadata_format = securesystemslib.keys.format_keyval_to_metadata(
+            keytype, scheme, key
+        )
+        keysinfo[keyid] = key_metadata_format
+        tuf.roledb.update_roleinfo(role, roleinfo, repository_name=self.name)
+
     def _add_target(self, targets_obj, file_path, custom=None):
         """
         <Purpose>
@@ -206,6 +231,40 @@ class Repository:
         normalize_file_line_endings(file_path)
         targets_obj.add_target(file_path, custom)
 
+    def add_metadata_key(self, role, pub_key_pem, scheme=DEFAULT_RSA_SIGNATURE_SCHEME):
+        """Add metadata key of the provided role.
+
+        Args:
+        - role(str): TUF role (root, targets, timestamp, snapshot or delegated one)
+        - pub_key_pem(str|bytes): Public key in PEM format
+
+        Returns:
+        None
+
+        Raises:
+        - securesystemslib.exceptions.FormatError: If the arguments are improperly formatted.
+        - securesystemslib.exceptions.UnknownRoleError: If 'rolename' has not been delegated by this
+                                                        targets object.
+        - securesystemslib.exceptions.UnknownKeyError: If 'key_id' is not found in the keydb database.
+
+        """
+        if isinstance(pub_key_pem, bytes):
+            pub_key_pem = pub_key_pem.decode("utf-8")
+
+        if is_delegated_role(role):
+            parent_role = self.find_delegated_roles_parent(role)
+            tuf.roledb._roledb_dict[self.name][role]["keyids"] = self.get_role_keys(
+                role, parent_role
+            )
+
+        key = import_rsakey_from_pem(pub_key_pem, scheme)
+        self._role_obj(role).add_verification_key(key)
+
+        if is_delegated_role(role):
+            keyids = tuf.roledb.get_roleinfo(role, self.name)["keyids"]
+            self.set_delegated_role_property("keyids", role, keyids, parent_role)
+            self._add_delegated_key(parent_role, keyids[-1], pub_key_pem, scheme=scheme)
+
     def _load_tuf_repository(self, path):
         """
         Load tuf repository. Should only be called directly if a different set of metadata files
@@ -215,7 +274,10 @@ class Repository:
         # to avoid errors raised by tuf
         if not self.targets_path.is_dir():
             self.targets_path.mkdir(parents=True, exist_ok=True)
-        self._tuf_repository = load_repository(path, self.name)
+        try:
+            self._tuf_repository = load_repository(path, self.name)
+        except RepositoryError:
+            raise InvalidRepositoryError(f"{self.name} is not a valid TUF repository!")
 
     def reload_tuf_repository(self):
         """
@@ -338,50 +400,46 @@ class Repository:
         - Set of all target paths relative to targets directory
         """
         all_roles = self.get_all_targets_roles()
+        return self.get_singed_target_files_of_roles(all_roles)
+
+    def get_singed_target_files_of_roles(self, roles):
+        """Return all target files signed by the specified roles
+
+        Args:
+        - roles whose target files will be returned
+
+        Returns:
+        - Set of paths of target files of a role relative to targets directory
+        """
+        if roles is None:
+            roles = self.get_all_targets_roles()
         return set(
             reduce(
                 operator.iconcat,
-                [self._role_obj(role).target_files for role in all_roles],
+                [self._role_obj(role).target_files for role in roles],
                 [],
             )
         )
 
-    def add_metadata_key(self, role, pub_key_pem, scheme=DEFAULT_RSA_SIGNATURE_SCHEME):
-        """Add metadata key of the provided role.
+    def get_singed_targets_with_custom_data(self, roles):
+        """Return all target files signed by the specified roles and and their custom data
+        as specified in the metadata files
 
         Args:
-        - role(str): TUF role (root, targets, timestamp, snapshot or delegated one)
-        - pub_key_pem(str|bytes): Public key in PEM format
+        - roles whose target files will be returned
 
         Returns:
-        None
-
-        Raises:
-        - securesystemslib.exceptions.FormatError: If the arguments are improperly formatted.
-        - securesystemslib.exceptions.UnknownRoleError: If 'rolename' has not been delegated by this
-                                                        targets object.
-        - securesystemslib.exceptions.UnknownKeyError: If 'key_id' is not found in the keydb database.
-
+        - A dictionary whose keys are parts of target files relative to the targets directory
+        and values are custom data dictionaries.
         """
-        if isinstance(pub_key_pem, bytes):
-            pub_key_pem = pub_key_pem.decode("utf-8")
-
-        if is_delegated_role(role):
-            parent_role = self.find_delegated_roles_parent(role)
-            tuf.roledb._roledb_dict[self.name][role]["keyids"] = self.get_role_keys(
-                role, parent_role
-            )
-
-        key = import_rsakey_from_pem(pub_key_pem, scheme)
-        self._role_obj(role).add_verification_key(key)
-
-        if is_delegated_role(role):
-            self.set_delegated_role_property(
-                "keyids",
-                role,
-                tuf.roledb.get_roleinfo(role, self.name)["keyids"],
-                parent_role,
-            )
+        if roles is None:
+            roles = self.get_all_targets_roles()
+        target_files = {}
+        for role in roles:
+            roles_targets = self._role_obj(role).target_files
+            for target_file, custom_data in roles_targets.items():
+                target_files.setdefault(target_file, {}).update(custom_data)
+        return target_files
 
     def modify_targets(self, added_data=None, removed_data=None):
         """Creates a target.json file containing a repository's commit for each repository.
@@ -1113,6 +1171,24 @@ class Repository:
             roles_targets_mapping.setdefault(role_name, []).append(target_filename)
         return roles_targets_mapping
 
+    def unmark_dirty_role(self, role):
+        """
+        Unmakes one dirty role. This means that the corresponding metadata file
+        will not be updated.
+        Args:
+        - role which should be unmaked
+        """
+        self.unmark_dirty_roles([role])
+
+    def unmark_dirty_roles(self, roles):
+        """
+        Unmakes dirty roles. This means that the corresponding metadata files
+        will not be updated.
+        Args:
+        - roles which should be unmaked
+        """
+        self._repository.unmark_dirty(roles)
+
     def update_role_keystores(
         self, role_name, signing_keys, start_date=None, interval=None, write=True
     ):
