@@ -67,7 +67,7 @@ TARGET1_SHA_MISMATCH = "Mismatch between target commits specified in authenticat
 TARGET2_SHA_MISMATCH = "Mismatch between target commits specified in authentication repository and target repository namespace/TargetRepo2"
 TARGETS_MISMATCH_ANY = "Mismatch between target commits specified in authentication repository and target repository"
 NO_WORKING_MIRRORS = f"Validation of authentication repository {AUTH_REPO_REL_PATH} failed due to error: No working mirror was found"
-TIMESTAMP_EXPIRED = "Metadata 'timestamp' expired"
+ROOT_EXPIRED = "Metadata 'root' expired"
 REPLAYED_METADATA = "ReplayedMetadataError"
 IS_A_TEST_REPO = f"Repository {AUTH_REPO_REL_PATH} is a test repository."
 NOT_A_TEST_REPO = f"Repository {AUTH_REPO_REL_PATH} is not a test repository."
@@ -241,11 +241,11 @@ def test_valid_update_no_auth_repo_one_invalid_target_repo_exists(
 
 
 def test_updater_expired_metadata(updater_repositories, origin_dir, client_dir):
-    # without using freeze_time, we expect to get timestamp expired error
+    # without using freeze_time, we expect to get metadata expired error
     repositories = updater_repositories["test-updater-valid"]
     clients_auth_repo_path = client_dir / AUTH_REPO_REL_PATH
     _update_invalid_repos_and_check_if_repos_exist(
-        client_dir, repositories, TIMESTAMP_EXPIRED, set_time=False
+        client_dir, repositories, ROOT_EXPIRED, set_time=False
     )
     # make sure that the last validated commit does not exist
     _check_if_last_validated_commit_exists(clients_auth_repo_path)
