@@ -78,7 +78,8 @@ def attach_to_group(group):
                   "the target repositories do not have remotes set")
     @click.option("--custom", default=None, help="A dictionary containing custom "
                   "targets info which will be added to repositories.json")
-    def generate_repositories_json(path, root_dir, namespace, targets_rel_dir, custom):
+    @click.option("--use-mirrors", is_flag=True, help="Whether to generate mirrors.json or not")
+    def generate_repositories_json(path, root_dir, namespace, targets_rel_dir, custom, use_mirrors):
         """
         Generate repositories.json. This file needs to be one of the authentication repository's
         target files or the updater won't be able to validate target repositories.
@@ -103,7 +104,13 @@ def attach_to_group(group):
         The url of a repository corresponds to its git remote url if set and to its location on the file
         system otherwise. Test repositories might not have remotes. If targets-rel-dir is specified
         and a repository does not have remote url, its url is calculated as a relative path to the
-        repository's location from this path.
+        repository's location from this path. There are two options of defining urls:
+        1. Directly specifying all urls of all repositories directly inside repositories.json
+        2. Using a separate mirrors.json file. This file will be generated only if use-mirrors flag is provided.
+        The mirrors file consists of a list of templates which contain namespace (org name) and repo name as arguments.
+        E.g. "https://github.com/{org_name}/{repo_name}.git". If a target repository's namespaced name
+        is namespace1/target_repo1, its url will be calculated by replacing {org_name} with namespace1
+        and {repo_name} with target_repo1 and would result in "https://github.com/namespace1/target_repo1.git"
 
         While urls are the only information that the updater needs, it is possible to add
         any other data using the custom option. Custom data can either be specified in a .json file
@@ -124,7 +131,9 @@ def attach_to_group(group):
         Note: this command does not automatically register repositories.json as a target file.
         It is recommended that the content of the file is reviewed before doing so manually.
         """
-        developer_tool.generate_repositories_json(path, root_dir, namespace, targets_rel_dir, custom)
+        developer_tool.generate_repositories_json(
+            path, root_dir, namespace, targets_rel_dir, custom, use_mirrors
+        )
 
     @repo.command()
     @click.argument("path")
@@ -140,6 +149,7 @@ def attach_to_group(group):
                   "the target repositories do not have remotes set")
     @click.option("--custom", default=None, help="A dictionary containing custom "
                   "targets info which will be added to repositories.json")
+    @click.option("--use-mirrors", is_flag=True, help="Whether to generate mirrors.json or not")
     @click.option("--add-branch", default=False, is_flag=True, help="Whether to add name of "
                   "the current branch to target files")
     @click.option("--keystore", default=None, help="Location of the keystore files")
@@ -150,7 +160,7 @@ def attach_to_group(group):
     @click.option("--test", is_flag=True, default=False, help="Indicates if the created repository "
                   "is a test authentication repository")
     @click.option("--scheme", default=DEFAULT_RSA_SIGNATURE_SCHEME, help="A signature scheme used for signing.")
-    def initialize(path, root_dir, namespace, targets_rel_dir, custom, add_branch, keystore,
+    def initialize(path, root_dir, namespace, targets_rel_dir, custom, use_mirrors, add_branch, keystore,
                    keys_description, commit, test, scheme):
         """
         \b
@@ -165,8 +175,8 @@ def attach_to_group(group):
         In order to have greater control over individual steps and be able to review files created
         in the initialization process, execute the mentioned commands separately.
         """
-        developer_tool.init_repo(path, root_dir, namespace, targets_rel_dir, custom, add_branch,
-                                 keystore, keys_description, commit, test, scheme)
+        developer_tool.init_repo(path, root_dir, namespace, targets_rel_dir, custom, use_mirrors,
+                                 add_branch, keystore, keys_description, commit, test, scheme)
 
     @repo.command()
     @click.argument("url")
@@ -183,7 +193,7 @@ def attach_to_group(group):
         """
         Update and validate local authentication repository and target repositories. Remote
         authentication's repository url needs to be specified when calling this command. If the
-        authentication repository and the target repositories are in the same root repository,
+        authentication repository and the target repositories are in the same root directory,
         locations of the target repositories are calculated based on the authentication repository's
         path. If that is not the case, it is necessary to redefine this default value using the
         --clients-root-dir option. This means that if authentication repository's path is
@@ -191,10 +201,10 @@ def attach_to_group(group):
         if clients-root-dir is not specified.
         Names of target repositories (as defined in repositories.json) are appended to the root repository's
         path thus defining the location of each target repository. If names of target repositories
-        are namespace/repo1, namespace/repo2 etc and the root directory is E:\\root, path of target
+        are namespace/repo1, namespace/repo2 etc and the root directory is E:\\root, path of the target
         repositories will be calculated as E:\\root\\namespace\\repo1, E:\\root\\namespace\\root2 etc.
 
-        If remote repository's url is a file system path, it is necessary yo call this command with
+        If remote repository's url is a file system path, it is necessary to call this command with
         --from-fs flag so that url validation is skipped. When updating a test repository (one that has
         the "test" target file), use --authenticate-test-repo flag. An error will be raised
         if this flag is omitted in the mentioned case. Do not use this flag when validating a non-test
